@@ -28,7 +28,7 @@ ssize_t getdelim(char ** lineptr, size_t * n,
 	ssize_t pos = -1;
 	int c;
 
-	if (lineptr || n || stream) {
+	if (lineptr && n && stream) {
 		if (!(buf = *lineptr)) { /* If passed NULL for buffer, */
 			*n = 0;		/* ignore value passed and treat size as 0. */
 		}
