@@ -138,10 +138,10 @@ static gboolean action_current(action * a)
     gint ret;
     gpointer stat_data;
 
-    if (current_image == NULL)
+    if (current_image == NULL || current_image->node == NULL)
         return FALSE;
 
-    stat_data = stat_loaded_files();
+    stat_data = stat_loaded_files(FALSE);
 
     ret = action_on_filename(a, current_image->node->data);
     process_system_retval(a, ret);
@@ -158,10 +158,12 @@ static gboolean action_every(action * a)
     gpointer stat_data;
     GlivImage *before = current_image;
 
-    stat_data = stat_loaded_files();
+    stat_data = stat_loaded_files(FALSE);
 
     for (node = get_list_head(); node != NULL; node = node->next) {
-        ret = action_on_filename(a, node->data) || ret;
+        gint new_ret = action_on_filename(a, node->data);
+        if (new_ret != 0)
+            ret = new_ret;
         process_events();
     }
 
@@ -475,8 +477,8 @@ static GtkListStore *make_tree_view(GtkWidget * dialog)
     return list_store;
 }
 
-static gboolean read_store(GtkTreeModel * model, GtkTreePath * path,
-                           GtkTreeIter * iter)
+static gboolean read_store(GtkTreeModel * unused_model,
+                           GtkTreePath * unused_path, GtkTreeIter * iter)
 {
     gchar *name, *command;
     action *a = g_new(action, 1);
