@@ -353,7 +353,7 @@ static gboolean process_arrow_key(GdkEventKey * event, gboolean press)
         move_or_rotate(event->state, x, y, angle);
     }
 
-    return FALSE;
+    return TRUE;
 }
 
 static gboolean key_press_event(GtkWidget * unused, GdkEventKey * event)
@@ -377,12 +377,10 @@ static gboolean key_press_event(GtkWidget * unused, GdkEventKey * event)
         load_direction(-1);
         break;
 
-    case GDK_minus:
     case GDK_KP_Subtract:
         zoom_in(1.0 / ZOOM_FACTOR);
         break;
 
-    case GDK_plus:
     case GDK_KP_Add:
     case GDK_equal:
         zoom_in(ZOOM_FACTOR);
@@ -393,7 +391,7 @@ static gboolean key_press_event(GtkWidget * unused, GdkEventKey * event)
         return process_arrow_key(event, TRUE);
     }
 
-    return FALSE;
+    return TRUE;
 }
 
 static gboolean key_release_event(GtkWidget * unused, GdkEventKey * event)
@@ -461,6 +459,13 @@ void install_callbacks(gpointer object)
                            G_CALLBACK(set_correct_cursor), NULL);
 }
 
+static gboolean finish(gpointer data)
+{
+    gboolean *b = data;
+    *b = TRUE;
+    return FALSE;
+}
+
 /*
  * We call process_events() when rebuilding the images menus and to wait for a
  * refresh.
@@ -469,13 +474,21 @@ void process_events(void)
 {
     GTimeVal beginning, now;
     gint delay = 0;
+    gboolean stop = FALSE;
 
+    if (!gtk_events_pending())
+        return;
+
+    g_idle_add(finish, &stop);
     g_get_current_time(&beginning);
 
-    while (gtk_events_pending() != 0 && delay < G_USEC_PER_SEC) {
+    while (!stop && delay < G_USEC_PER_SEC / 10 && gtk_events_pending()) {
         gtk_main_iteration_do(FALSE);
         g_get_current_time(&now);
         delay = (now.tv_sec - beginning.tv_sec) * G_USEC_PER_SEC +
             now.tv_usec - beginning.tv_usec;
     }
+
+    if (!stop)
+        g_idle_remove_by_data(&stop);
 }
