@@ -179,7 +179,6 @@ static GtkProgressBar *progress;
 static GtkWindow *progress_window;
 static GtkLabel *elapsed_label, *total_label, *remaining_label;
 static time_t start_time;
-static gboolean is_saving;      /* loading or saving ? */
 
 static gboolean set_true(gboolean * ptr)
 {
@@ -207,13 +206,15 @@ static GtkLabel *add_time_display(GtkTable * table, const gchar * text,
     return time_label;
 }
 
-static void create_progress_dialog(const gchar * filename, gpointer cancel_data)
+static void create_progress_dialog(const gchar * filename, gpointer cancel_data,
+                                   gboolean is_saving)
 {
     GtkButton *cancel;
     GtkTable *table;
     gchar *title;
 
     file_label = GTK_LABEL(gtk_label_new(NULL));
+    gtk_label_set_ellipsize(file_label, PANGO_ELLIPSIZE_MIDDLE);
     gtk_widget_show(GTK_WIDGET(file_label));
 
     ratio_label = GTK_LABEL(gtk_label_new(NULL));
@@ -242,10 +243,6 @@ static void create_progress_dialog(const gchar * filename, gpointer cancel_data)
 
     gtk_widget_show(GTK_WIDGET(table));
 
-    progress_window = GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
-    g_signal_connect_swapped(progress_window, "delete-event",
-                             G_CALLBACK(set_true), cancel_data);
-
     if (is_saving)
         title = g_strdup_printf(_("Saving collection: %s"),
                                 filename_to_utf8(filename));
@@ -253,9 +250,12 @@ static void create_progress_dialog(const gchar * filename, gpointer cancel_data)
         title = g_strdup_printf(_("Loading collection: %s"),
                                 filename_to_utf8(filename));
 
-    gtk_window_set_title(progress_window, title);
+    progress_window = new_window(title);
     g_free(title);
 
+    g_signal_connect_swapped(progress_window, "delete-event",
+                             G_CALLBACK(set_true), cancel_data);
+
     gtk_container_add(GTK_CONTAINER(progress_window), GTK_WIDGET(table));
     gtk_widget_show(GTK_WIDGET(progress_window));
     start_time = time(NULL);
@@ -354,7 +354,6 @@ static void update_progress(const gchar * filename, gint id, gint count)
 {
     const gchar *base;
     time_t current_time, elapsed_time, total_time, remaining_time;
-    size_t len;
     gchar *ratio_text;
 
     if (progress_window == NULL) {
@@ -379,20 +378,7 @@ static void update_progress(const gchar * filename, gint id, gint count)
         else
             base++;
 
-        len = strlen(base);
-        if (len > MAX_DISPLAYED_NAME_LENGTH) {
-            gchar *begin, *text;
-            const gchar *end;
-
-            begin = g_strndup(base, MAX_DISPLAYED_NAME_LENGTH / 2 - 3);
-            end = base + len - MAX_DISPLAYED_NAME_LENGTH / 2 + 3;
-            text = g_strconcat(begin, " ... ", end, NULL);
-            g_free(begin);
-
-            gtk_label_set_text(file_label, text);
-            g_free(text);
-        } else
-            gtk_label_set_text(file_label, base);
+        gtk_label_set_text(file_label, base);
     }
 
     ratio_text = g_strdup_printf("%d / %d", id, count - 1);
@@ -585,7 +571,6 @@ gint save_collection(FILE * file, gboolean * cancel)
     gboolean ok = FALSE;
 
     list = get_list_head();
-    is_saving = TRUE;
     work = g_new(serialization_data, 1);
     work->file = file;
     work->file_id = 0;
@@ -686,7 +671,7 @@ gboolean serialize_collection_gui(void)
         return FALSE;
     }
 
-    create_progress_dialog(filename, &cancel);
+    create_progress_dialog(filename, &cancel, TRUE);
 
     res = save_collection(file, &cancel);
     if (res) {
@@ -971,7 +956,6 @@ gint load_dot_gliv_from_file(const gchar * filename, FILE * file)
     struct coll_src *source;
     gboolean saved_force_load;
 
-    is_saving = FALSE;
     source = build_source(file);
 
     CHECK_MAGIC(source, "GLiv <COLLECTION>");
@@ -982,7 +966,7 @@ gint load_dot_gliv_from_file(const gchar * filename, FILE * file)
         goto error;
 
     if (rt)
-        create_progress_dialog(filename, &cancel);
+        create_progress_dialog(filename, &cancel, FALSE);
 
     items = g_new(tree_item *, count);
     for (i = 0; i < count; i++) {
@@ -1099,6 +1083,6 @@ gboolean deserialize_collection(void)
     if (nb_inserted <= 0)
         return FALSE;
 
-    open_next_image(nb_inserted == 1);
+    new_images(nb_inserted);
     return FALSE;
 }
