@@ -44,7 +44,7 @@ static GList *free_item(GList * item)
     next = item->next;
     count--;
 
-    g_list_delete_link(list, item);
+    list = g_list_delete_link(list, item);
     return next;
 }
 
