@@ -7,18 +7,18 @@
 void after_reorder(void);
 const gchar *get_image_notice(void);
 void load_direction(gint dir);
-void open_next_image(gboolean keep_next);
-GlivImage *load_first_image(void);
-void load_second_image(void);
+void load_first_image(void);
+void reload_current_image(void);
 void reload_images(void);
 gboolean menu_load(const gchar * filename);
 void unload(GList * node);
 void unload_images(void);
-gpointer stat_loaded_files(void);
+gpointer stat_loaded_files(gboolean quiet);
 void reload_changed_files(gpointer * stat_data);
 void set_slide_show_menu(GtkMenuItem * item);
 void start_slide_show(void);
 gboolean slide_show_started(void);
 gboolean toggle_slide_show(void);
+void new_images(gint nb_inserted);
 
 #endif
