@@ -351,7 +351,7 @@ static GlivImage *load_in_direction(gint dir)
     if (im != NULL) {
         im->node = node;
         im->number = id;
-    } else if (get_list_head() == NULL && current_image->node)
+    } else if (get_list_head() == NULL && current_image)
         /* No more images, not even the current one */
         current_image->node = NULL;
 
@@ -500,8 +500,12 @@ void load_first_image(void)
 
     install_segv_handler();
 
-    if (get_list_head() == NULL)
+    if (get_list_head() == NULL) {
+        if (options->build_menus)
+            /* Initialize the menus' timestamps for cond_rebuild to work */
+            rebuild_images_menus();
         return;
+    }
 
     first_image = load_in_direction(1);
     if (first_image == NULL)
