@@ -27,7 +27,6 @@
 #include "options.h"
 #include "tree.h"
 #include "thread.h"
-#include "images_menus.h"
 #include "callbacks.h"
 
 extern options_struct *options;
@@ -176,7 +175,8 @@ gboolean fill_thumbnail(tree_item * item)
     if (item->thumb == NULL)
         return FALSE;
 
-    g_hash_table_insert(thumbs_hash_table, item->thumb_key, item->thumb);
+    g_hash_table_insert(thumbs_hash_table, g_strdup(item->thumb_key),
+                        item->thumb);
     return TRUE;
 }
 
