@@ -33,7 +33,6 @@
 #include "rendering.h"
 #include "options.h"
 #include "loading.h"
-#include "callbacks.h"
 #include "messages.h"
 
 /*
@@ -81,7 +80,7 @@ static void draw_image(GlivImage * im, gfloat * matrix, gfloat opacity)
 }
 
 /* res = big - small */
-static void diff_timeval(GTimeVal * res, GTimeVal * big, GTimeVal * small)
+void diff_timeval(GTimeVal * res, GTimeVal * big, GTimeVal * small)
 {
     res->tv_sec = big->tv_sec - small->tv_sec;
     if (big->tv_usec >= small->tv_usec)
@@ -92,24 +91,20 @@ static void diff_timeval(GTimeVal * res, GTimeVal * big, GTimeVal * small)
     }
 }
 
-static void sleep_remaining(GTimeVal * started, GTimeVal * required)
+static void transition_single_step(GdkGLDrawable * gldrawable,
+                                   GlivImage * im1, gfloat * matrix1,
+                                   GlivImage * im2, gfloat * matrix2,
+                                   gfloat alpha)
 {
-    GTimeVal now;
-    GTimeVal done;
-    GTimeVal sleep_time;
-    glong delay;
+    clear();
 
-    g_get_current_time(&now);
+    draw_image(im1, matrix1, 1.0 - alpha);
+    draw_image(im2, matrix2, alpha);
 
-    diff_timeval(&done, &now, started);
-    diff_timeval(&sleep_time, required, &done);
-
-    delay = sleep_time.tv_sec * G_USEC_PER_SEC + sleep_time.tv_usec;
-    if (delay > 0)
-        g_usleep(delay);
+    gdk_gl_drawable_swap_buffers(gldrawable);
+    gdk_gl_drawable_wait_gl(gldrawable);
 }
 
-
 /*
  * This is the transition used if both images lack an alpha channel and the
  * background color is black.
@@ -118,61 +113,49 @@ static void sleep_remaining(GTimeVal * started, GTimeVal * required)
 static void transition_no_alpha_black(GlivImage * im1, gfloat * matrix1,
                                       GlivImage * im2, gfloat * matrix2)
 {
-    gint sleep, step;
-    gfloat alpha;
-    GTimeVal sleeping_time;
-    GTimeVal now;
+    gint steps = 0;
+    GTimeVal now, transition_start;
+    gint advance = 0;
     GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(gl_widget);
-#if FPS_COUNTER
-    GTimer *fps_counter;
-#endif
 
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
 
-    if (options->trans_sleep < 0)
-        sleep = 0;
-    else
-        sleep = options->trans_sleep;
+    g_get_current_time(&transition_start);
+    g_get_current_time(&now);
 
-    sleeping_time.tv_sec = sleep / 1000;
-    sleeping_time.tv_usec = (sleep % 1000) * 1000;
+    for (;;) {
+        gint delay = diff_timeval_us(&now, &transition_start);
+        gfloat alpha = (gfloat) delay / (options->trans_time * 1000);
 
-#if FPS_COUNTER
-    fps_counter = g_timer_new();
-#endif
-
-    for (step = 0; step < options->trans_steps; step++) {
-        clear();
-        alpha = (gfloat) (step + 1) / (options->trans_steps + 1);
-
-        draw_image(im1, matrix1, 1.0 - alpha);
-        draw_image(im2, matrix2, alpha);
+        if (alpha > 1.0)
+            break;
 
         g_get_current_time(&now);
 
-        gdk_gl_drawable_swap_buffers(gldrawable);
-        gdk_gl_drawable_wait_gl(gldrawable);
+        transition_single_step(gldrawable, im1, matrix1, im2, matrix2, alpha);
+
+        if (options->fps > 0) {
+            if (get_fps_limiter_delay(&now) > 0)
+                gtk_main_iteration_do(FALSE);
+            delay = get_fps_limiter_delay(&now) + advance;
+            if (delay > 0)
+                g_usleep(delay);
+            advance += get_fps_limiter_delay(&now);
 
 #if FPS_COUNTER
-        g_timer_stop(fps_counter);
-#endif
-
-        process_events();
-        sleep_remaining(&now, &sleeping_time);
-
-#if FPS_COUNTER
-        g_timer_continue(fps_counter);
+            printf("transition lag: %dus\n", -get_fps_limiter_delay(&now));
 #endif
+        }
+        steps++;
     }
 
 #if FPS_COUNTER
-    if (fps_counter != NULL) {
-        gdouble seconds = g_timer_elapsed(fps_counter, NULL);
-        g_timer_destroy(fps_counter);
-        printf("%dx%d -> %dx%d : %d frames in %lg sec => %lg fps\n",
+    {
+        gfloat usecs = (gfloat) diff_timeval_us(&now, &transition_start);
+        printf("%dx%d -> %dx%d : %d frames in %f sec => %f fps\n",
                im1->width, im1->height, im2->width, im2->height,
-               options->trans_steps, seconds, options->trans_steps / seconds);
+               steps, usecs / G_USEC_PER_SEC, G_USEC_PER_SEC * steps / usecs);
     }
 #endif
 
@@ -214,7 +197,10 @@ static GlivImage *grab_image(GlivImage * im, gfloat * matrix,
 
     /* Area where we take the screenshot */
 #if NO_OPENGL
-    x = 0; y = 0; w = 640; h = 480;
+    x = 0;
+    y = 0;
+    w = 640;
+    h = 480;
 #else
     glGetIntegerv(GL_VIEWPORT, viewport);
     x = viewport[0];
