@@ -39,6 +39,7 @@
 #include "menus.h"
 #include "cursors.h"
 #include "loading.h"
+#include "help.h"
 
 extern rt_struct *rt;
 extern options_struct *options;
@@ -62,9 +63,14 @@ static GtkEntry *entry_state;
  * Called when resizing the window because the image changed or a
  * widget has been toggled.
  */
-static void resize_window(gint width, gint height, gboolean force)
+static void resize_window(gint width, gint height, gboolean first_time)
 {
-    if (options->resize_win == FALSE && force == FALSE)
+    GtkRequisition req;
+
+    if (options->resize_win == FALSE && first_time == FALSE)
+        return;
+
+    if (first_time && options->initial_geometry)
         return;
 
     width = MAX(width, 320);
@@ -78,9 +84,8 @@ static void resize_window(gint width, gint height, gboolean force)
      * width x height widget,
      */
     gtk_widget_set_size_request(gl_widget, width, height);
-    gtk_window_resize(main_window, 1, 1);
-
-    process_events();
+    gtk_widget_size_request(GTK_WIDGET(main_window), &req);
+    gtk_window_resize(main_window, req.width, req.height);
 
     /* then, we make the widget downsizable. */
     gtk_widget_set_size_request(gl_widget, 1, 1);
@@ -287,9 +292,9 @@ void update_status_bar(void)
     gfloat zoom, angle;
 
     if (current_image == NULL)
-        return;
-
-    if (current_image->ident != NULL) {
+        /* Loading first image */
+        set_ident_text("");
+    else if (current_image->ident != NULL) {
         /* Filename and dimensions status, only the first time. */
         set_ident_text(current_image->ident);
         g_free(current_image->ident);
@@ -345,11 +350,48 @@ static gboolean first_map(void)
     return FALSE;
 }
 
+static void set_initial_geometry(void)
+{
+    GdkGeometry size_hints = {
+        1, 1, 0, 0, 1, 1, 1, 1, 0.0, 0.0, GDK_GRAVITY_NORTH_WEST
+    };
+
+    if (!options->initial_geometry)
+        goto fail;
+
+    if (options->fullscreen) {
+        g_printerr(_("Ignoring initial geometry (%s) as"
+                     " fullscreen mode is requested\n"),
+                   options->initial_geometry);
+        goto fail;
+    }
+
+    gtk_window_set_geometry_hints(main_window, GTK_WIDGET(main_window),
+                                  &size_hints,
+                                  GDK_HINT_MIN_SIZE | GDK_HINT_BASE_SIZE |
+                                  GDK_HINT_RESIZE_INC);
+
+    if (!gtk_window_parse_geometry(main_window, options->initial_geometry)) {
+        g_printerr(_("Cannot parse geometry: %s\n"), options->initial_geometry);
+        goto fail;
+    }
+
+    return;
+
+  fail:
+    g_free(options->initial_geometry);
+    options->initial_geometry = NULL;
+}
+
 void create_windows(void)
 {
     GtkWindow *first_window;
     GtkAccelGroup *accel_group;
     GtkHBox *hbox;
+    GdkPixbuf *logo = get_gliv_logo();
+
+    if (logo != NULL)
+        gtk_window_set_default_icon(logo);
 
     rt->scr_width = gdk_screen_width();
     rt->scr_height = gdk_screen_height();
@@ -406,6 +448,8 @@ void create_windows(void)
 
     gtk_container_add(GTK_CONTAINER(first_window), GTK_WIDGET(widgets));
     g_signal_connect_after(first_window, "map", G_CALLBACK(first_map), NULL);
+    gtk_widget_show_all(GTK_WIDGET(widgets));
+    set_initial_geometry();
     gtk_widget_show_all(GTK_WIDGET(first_window));
 }
 
@@ -413,7 +457,7 @@ void create_windows(void)
  * Called when going from fullscreen mode to window mode, or when
  * the displayed image changed in window mode.
  */
-void goto_window(GlivImage * im, gboolean force)
+void goto_window(GlivImage * im, gboolean first_time)
 {
     gint new_width, new_height;
 
@@ -421,8 +465,8 @@ void goto_window(GlivImage * im, gboolean force)
         im = current_image;
 
     if (im == NULL) {
-        new_width = 320;
-        new_height = 240;
+        new_width = 0;
+        new_height = 0;
     } else {
         new_width = im->width;
         new_height = im->height;
@@ -434,7 +478,7 @@ void goto_window(GlivImage * im, gboolean force)
     if (new_height >= rt->scr_height)
         new_height = 3 * rt->scr_height / 4;
 
-    resize_window(new_width, new_height, force);
+    resize_window(new_width, new_height, first_time);
 }
 
 /*
@@ -487,12 +531,17 @@ void toggle_fullscreen(gboolean enable)
 void update_window_title(void)
 {
     const gchar *path;
-    gchar *basename;
+    gchar *basename, *title;
+
+    if (current_image->node->data == NULL)
+        return;
 
     path = filename_to_utf8(current_image->node->data);
     basename = g_path_get_basename(path);
-    gtk_window_set_title(main_window, basename);
+    title = g_strdup_printf("GLiv - %s", basename);
+    gtk_window_set_title(main_window, title);
     g_free(basename);
+    g_free(title);
 }
 
 GtkWindow *get_current_window(void)
@@ -502,3 +551,47 @@ GtkWindow *get_current_window(void)
 
     return NULL;
 }
+
+GtkWindow *new_window(const gchar * title)
+{
+    GtkWindow *win = GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
+    GtkWindow *parent = get_current_window();
+
+    gtk_window_set_title(win, title);
+    gtk_window_set_transient_for(win, parent);
+    gtk_window_set_destroy_with_parent(win, TRUE);
+    gtk_window_set_type_hint(win, GDK_WINDOW_TYPE_HINT_DIALOG);
+
+    return win;
+}
+
+gboolean toggle_floating_windows(void)
+{
+    static GList *hidden_windows;
+    GList *ptr;
+
+    if (hidden_windows) {
+        for (ptr = hidden_windows; ptr; ptr = ptr->next) {
+            gtk_widget_show(GTK_WIDGET(ptr->data));
+            g_object_unref(ptr->data);
+        }
+
+        g_list_free(hidden_windows);
+        hidden_windows = NULL;
+    } else {
+        GList *toplevels = gtk_window_list_toplevels();
+
+        /* First two windows are the main window and the full screen one */
+        for (ptr = toplevels->next->next; ptr; ptr = ptr->next) {
+            GtkWidget *widget = GTK_WIDGET(ptr->data);
+            if (GTK_WIDGET_VISIBLE(widget)) {
+                gtk_widget_hide(widget);
+                hidden_windows = g_list_prepend(hidden_windows, widget);
+                g_object_ref(widget);
+            }
+        }
+        g_list_free(toplevels);
+    }
+
+    return FALSE;
+}
