@@ -22,13 +22,12 @@
  * The zoom frame with funky colors *
  ************************************/
 
-#include <gtk/gtkgl.h>
-
 #include "gliv.h"
 #include "zoom_frame.h"
 #include "options.h"
 #include "rendering.h"
 #include "matrix.h"
+#include "opengl.h"
 
 extern rt_struct *rt;
 extern GtkWidget *gl_widget;
